/////////////////////////////////////////////////////////////////////////////
//
/// @file typedefs.h
///
/// @author Tomasz Kapela   @date 2010-04-08
//
/////////////////////////////////////////////////////////////////////////////


#ifndef _CAPD_KAPELA_TYPEDEFS_H_
#define _CAPD_KAPELA_TYPEDEFS_H_

//#define _USE_DOUBLES_
//#define __HAVE_MPFR__

#include "capd/capdlib.h"
#include "capd/mpcapdlib.h"
const int G_rank = 3;
const int G_dim = 0;
#ifndef _USE_DOUBLES_
typedef capd::MpFloat Float;
typedef capd::MpInterval Interval;

#else
typedef double Float;
//typedef capd::Interval Interval;
typedef capd::intervals::Interval<double> Interval;
//typedef capd::intervals::Interval<long double> Interval;
#include "capd/map/Function.hpp"
#include "capd/dynset/C0Rect2Set.hpp"
#endif

#define _USE_CAPDLIB_
#ifdef _USE_CAPDLIB_
typedef capd::DVector DVector;
typedef capd::DMatrix DMatrix;
/// -------- Non rigorous types ------------
typedef capd::MpMatrix Matrix;
typedef capd::MpVector Vector;
typedef capd::map::Map<Matrix> Map;
typedef Map::JetType CnCoeff;
typedef Map::FunctionType Function;

/// -------- Rigorous types ----------------
typedef capd::MpIMatrix IMatrix;
typedef capd::MpIVector IVector;
typedef capd::MpIMap IMap;
typedef IMap::FunctionType IFunction;
typedef capd::MpITaylor ITaylor;
typedef capd::MpITaylor ITaylorHOE;
typedef ITaylor IC1Taylor;
typedef capd::MpICnTaylor ICnTaylor;

typedef capd::poincare::TimeMap<ITaylor> ITimeMap;

typedef capd::MpPoincareMap PoincareMap;
typedef capd::MpIPoincareMap IPoincareMap;
typedef capd::MpICnPoincareMap ICnPoincareMap;

typedef capd::MpC0Rect2Set C0Set;
typedef capd::MpC1Rect2Set C1Set;
typedef capd::MpCnRect2Set CnSet;

typedef capd::vectalg::EuclLNorm<IVector, IMatrix> IEuclLNorm;

typedef IMap::JetType ICnCoeff;

#else

#include "capd/multiPrec/MpReal.h"
#include "capd/intervals/Interval.hpp"
#include "capd/vectalg/Vector.hpp"
#include "capd/vectalg/Matrix.hpp"
#include "capd/map/CnMap.hpp"
#include "capd/dynsys/Taylor.hpp"
#include "capd/dynsys/TaylorHOE.hpp"
#include "capd/dynsys/CnTaylor.hpp"
#include "capd/dynsys/StepControl.h"
#include "capd/dynset/C0Rect2RSet.h"
#include "capd/dynset/C1Rect2RSet.h"
#include "capd/dynset/CnRect2.hpp"
#include "capd/vectalg/Norm.hpp"
#include "capd/rounding/DoubleRounding.h"
#include "capd/map/CnCoeffSlice.h"
#include "capd/intervals/MpInterval.h"
//using capd::intervals::atan2;
#include "capd/intervals/IComplex.h"
#include "capd/poincare/TimeMap.hpp"
#include "capd/auxil/logger.h"
#include "capd/poincare/PoincareMap.hpp"
#include "capd/poincare/BasicPoincareMapJet.hpp"
#include "capd/filib/Interval.h"
typedef capd::vectalg::Vector<double, 0> DVector;
typedef capd::vectalg::Matrix<double, 0, 0> DMatrix;
/// -------- Non rigorous types ------------
typedef capd::vectalg::Matrix<Float, G_dim, G_dim> Matrix;
typedef capd::vectalg::Vector<Float, G_dim> Vector;
typedef capd::map::CnMap<Matrix, G_rank> Map;
typedef Map::CnCoeffType CnCoeff;
typedef Map::FunctionType Function;
typedef capd::dynsys::BasicTaylor<Map, capd::dynsys::MpLastTermsStepControl<Float> > Taylor;
//typedef capd::dynsys::Taylor<Map, capd::dynsys::FixedStepControl<Interval> > TaylorNoControl;
//typedef capd::dynsys::TaylorHOE<Map> TaylorHOE;

typedef capd::poincare::BasicPoincareMap<Taylor> PoincareMap;
typedef capd::poincare::BasicPoincareMapJet<Taylor> PoincareMapJet;

/// -------- Rigorous types ----------------
typedef capd::vectalg::Matrix<Interval, G_dim, G_dim> IMatrix;
typedef capd::vectalg::Vector<Interval, G_dim> IVector;
typedef capd::map::CnMap<IMatrix, G_rank> IMap;
typedef IMap::FunctionType IFunction;
//typedef capd::dynsys::NoStepControl StepControl;
typedef capd::dynsys::IEncFoundStepControl StepControl;
//typedef capd::dynsys::ILastTermsStepControl StepControl;
typedef capd::dynsys::Taylor<IMap, capd::dynsys::IEncFoundStepControl> ITaylor;
typedef capd::dynsys::Taylor<IMap, capd::dynsys::FixedStepControl<Interval> > ITaylorNoControl;
typedef capd::dynsys::TaylorHOE<IMap> ITaylorHOE;
typedef ITaylor IC1Taylor;
typedef capd::dynsys::CnTaylor<IMap, capd::dynsys::IEncFoundStepControl> ICnTaylor;

typedef capd::poincare::TimeMap<ITaylor> ITimeMap;
typedef capd::poincare::TimeMap<ITaylorNoControl> ITimeMapNoControl;
typedef capd::poincare::TimeMap<ICnTaylor> ICnTimeMap;
typedef capd::poincare::TimeMap<ITaylorHOE> ITimeMapHOE;

typedef capd::poincare::PoincareMap<ITaylor> IPoincareMap;
typedef capd::poincare::PoincareMap<IC1Taylor> IC1PoincareMap;
typedef capd::poincare::PoincareMap<ICnTaylor> ICnPoincareMap;
typedef capd::poincare::PoincareMap<ITaylorHOE> IPoincareMapHOE;

typedef capd::dynset::C0Rect2RSet<IMatrix> C0Set;
typedef capd::dynset::C1Rect2RSet<IMatrix> C1Set;
typedef capd::dynset::CnRect2<IMatrix> CnSet;

typedef capd::vectalg::EuclLNorm<IVector, IMatrix> IEuclLNorm;

typedef IMap::CnCoeffType ICnCoeff;
typedef capd::map::CnCoeffSlice<IMatrix> ICnCoeffSlice;
#endif

//#define USE_STD_COMPLEX 1
#ifdef USE_STD_COMPLEX

#include <complex>
#include "capd/intervals/IComplex.h"
  typedef std::complex<Float> Complex;
  typedef std::complex<Interval> IComplex;
#else 

  #include "capd/intervals/Complex.h"
  /// ------- Complex types ----------------
  typedef capd::intervals::Complex<Float> Complex;
  typedef capd::intervals::Complex<Interval> IComplex;
#endif

typedef capd::vectalg::Vector<IComplex, G_dim> CIVector;
typedef capd::vectalg::Matrix<IComplex, G_dim, G_dim> CIMatrix;
typedef capd::map::Map<CIMatrix> CIMap;
typedef CIMap::JetType CICnCoeff;

using capd::vectalg::Multipointer;
using capd::vectalg::Multiindex;


#endif // _CAPD_KAPELA_TYPEDEFS_H_
