/**
 *  @file h4_stability.cpp
 *
 *  @author kapela  @date 2010-11-25
 */

#include "typedefs.h"
//#include "h4data.h"
//#include "H4Parallel.h"
#include "capd/auxil/Stopwatch.h"
#include "capd/auxil/logger.h"
#include "capd/auxil/arg.h"
//#include "toText.h"
//#include "PoincareMapComputation.h"
#include "normalFormComputation.h"
#include "capd/normalForms/normalForms.h"
#include "IniParser.h"
#include "ParallelMap.h"
#include "CnCoeffUtils.h"
#include "capd/basicalg/TexWriter.h"
//#include "capd/map/CnCoeff.hpp"
//#include "capd/vectalg/Vector.hpp"
#include "ioUtils.h"

#include <iomanip>

const char * resultFile = "report.txt";
const char * consoleFile = "console.txt";



const char
    *helpinfo =
        "Usage:"
        "\n  progName dataFile [options]\n\n"
"Options are:\n"
"-h              to display this help information,\n\
-p --precision   precision in mantisa bits,\n\
--sout           file name where results will be stored,\n\
--scon           file name where console output will be placed,\n\
--tex            file where data in tex format will be placed \n\
-q               quiet mode, scon output is not showed on screen.\n\
-v --verbose     verbose mode - displays additional informations\n"
"--debug         debug mode\n";

int setUpParameters(
    int argc, char ** argv,
    std::string & dataFile,
    std::string & texFile
   ) {
  using namespace capd::auxil;
  int prec = 300;
  capd::scon.show = true;
  capd::sout.show = true;
  capd::sbug.show = false;
  arguments a;
  argbreak(a, "h");
  argoblig(a, 0, dataFile);
  arg(a, 0, resultFile);
  arg(a, "p", prec);
  arg(a, "-precision", prec);
  arg(a, "-sout", resultFile);
  arg(a, "-scon", consoleFile);
  arg(a, "-tex", texFile);
  argswitch(a, "q", capd::scon.show, false);
  argswitch(a, "v", capd::sbug.show, true);
  argswitch(a, "-verbose", capd::sbug.show, true);
  argswitch(a, "-debug", capd::sbug.show, true);

  // analyze the command line
  //  argstreamprepare (a);
  int argresult = a. analyze(argc, argv);
  //  argstreamset ();
  if(argresult < 0) {
    capd::serr << "\n ERROR when reading parameters";
  } else if(argresult > 0) {
    capd::sout << helpinfo << std::endl;
  } else {
    capd::scon.logfile(consoleFile);
    capd::sout.logfile(resultFile);
    setPrecision(prec);
    capd::sbug << "\n Precision used : " << prec ;
  }
  return argresult;
}

void readAndOutputODEParam(std::istream & file, capd::TexWriter & out){
  IniParser ini(file);
  Interval h, M;
  int N;
  IVector x0;
  getODEParameters(ini, h, M, N, x0);
  IVector x(2, x0.begin());
  out.precision(16);
  out << "\n \\[h = " << leftBound(h) << "\\]";
  out << "\n\n diam = $" << rightBound(size(x)) <<"$\n\n";
  out << "\n\n \\[ x = " <<  x << "\\]\n";
  out.setVectorStyle(2) << "\n\n \\[ x = " <<  x << "\\]\n" ;
  out.setVectorStyle(0);
}

int main(int argc, char ** argv) {
  try {

    capd::sout << " H4 KAM STABILITY\n\n";
    std::string dataFile = "", resultFile = "h4_result_kam.txt";

    if(setUpParameters(argc, argv, dataFile, resultFile))
      return 0;

    std::ifstream file(dataFile.c_str());
    if(!file){
      std::cerr << "\n Error opening file " << dataFile.c_str() << std::endl;
      return 1;
    }
    ICnCoeff cncoeff;
    file >> cncoeff;

    std::ifstream out(resultFile.c_str());
    ICnCoeff cncoeff2;
    //capd::normalForms::derivativesToSeries(coeff);
    out >> cncoeff2 ;
    capd::normalForms::seriesToDerivatives(cncoeff2);

    ICnCoeff c = cncoeff - cncoeff2;

    std::cout << "\n coeff 1 : "  << cncoeff
                  << "\n coeff 2 : " << cncoeff2
                  << "\n result  : " << c;

    std::cout << "\n coeff 1 : "  << cncoeff.toString()
              << "\n coeff 2 : " << cncoeff2.toString()
              << "\n result  : " << c.toString();

    int dim =  cncoeff.dimension();
      for(int i =0; i<dim; ++i){
       for(int order=0; order <= cncoeff.rank(); ++order){
        Multipointer mp = cncoeff.first(order);
        do{
           c(i,mp) = capd::intervals::intervalHull(cncoeff(i,mp), cncoeff2(i,mp));
        }while(cncoeff.next(mp));
       }
      }
      std::cout     << "\n result 2: " << c.toString() << "\n\n" <<c;


  } catch(std::exception &e) {
    capd::sout << e.what();
    return 1;
  }
  return 0;
}
